<?php defined('SYSPATH') OR die('No direct access allowed.');
/**
 * Synergy controller
 *
 *  op5, and the op5 logo are trademarks, servicemarks, registered servicemarks
 *  or registered trademarks of op5 AB.
 *  All other trademarks, servicemarks, registered trademarks, and registered
 *  servicemarks mentioned herein may be the property of their respective owner(s).
 *  The information contained herein is provided AS IS with NO WARRANTY OF ANY
 *  KIND, INCLUDING THE WARRANTY OF DESIGN, MERCHANTABILITY, AND FITNESS FOR A
 *  PARTICULAR PURPOSE.
 */
class Synergy_Controller extends Authenticated_Controller
{
	const MODE_VIEW = 1;
	const MODE_EDIT = 2;

	protected function _initHead()
	{
		$this->template->content->authorized_for_edit = nacoma::allowed();
		// This might be used later. Class nacoma is located in nacoma helper in ninja.
		// $this->template->content->authorized_for_export = nacoma::exportAllowed();
		$this->template->js_header = $this->add_view('js_header');
		$this->template->js_strings = "var synergy_data = " . json_encode(Synergy_Model::getStates()) . ";\n var actions = " . Synergy_Model::getActionsJson() . ";\n";

		$this->template->js_strings .= "var nagios = ".Synergy_Model::getNagiosObjectsJson().";\n";
		$this->template->css_header = '<link type="text/css" rel="stylesheet" href="' . url::base() . 'modules/synergy/views/synergy/css/synergy.css" />';
		$synergy_js_path = 'modules/synergy/js/';
		$this->xtra_js[] = $synergy_js_path.'synergy.json2.js';
		$this->xtra_js[] = $synergy_js_path.'synergy.jquery.js';
		$this->xtra_js[] = $synergy_js_path.'synergy.liquidmetal.js';
		$this->xtra_js[] = $synergy_js_path.'synergy.flexselect.js';
		$this->xtra_js[] = $synergy_js_path.'synergy.tipsy.js';
		$this->xtra_js[] = $synergy_js_path.'synergy.colorbox.js';
		$this->xtra_js[] = $synergy_js_path.'synergy.validate.js';
		$this->xtra_js[] = $synergy_js_path.'synergy.address.js';
		$this->xtra_js[] = $synergy_js_path.'synergy.js';
	}

	public function view()
	{
		$this->template->content = $this->add_view('synergy/index');
		$this->template->content->mode = self::MODE_VIEW;
		$this->_initHead();
		$this->template->title = _('Monitoring » Business Services » View');
		$this->template->disable_refresh = true;
	}

	public function edit()
	{
		$this->template->content = $this->add_view('synergy/index');
		$this->template->content->mode = self::MODE_EDIT;
		$this->_initHead();
		$this->template->title = _('Monitoring » Business Services » Edit');
		$this->template->disable_refresh = true;
	}

	/**
	 * Fetches states for all trees
	 */
	public function update()
	{
		$result = array(
			"status" => 200,
			"data" => array()
		);
		$trees = Synergy_Model::getStates();
		$result["data"]["tree"] = $trees;
		print(json_encode($result));
	}

	/**
	 * Checks whether rules exist
	 * @return string
	 */
	public function rule_check()
	{
		$this->auto_render = false;
		$result = array("status" => 200, "data" => false);
		try {
			if (!isset($_GET["rule"])) {
				throw new Exception("You have pass a rule argument");
			}
			$rule = (string) $_GET["rule"];
			if (strlen($rule) === 0) {
				throw new Exception("Illegal rule argument");
			}
			$result["data"] = Synergy_Model::ruleExists($rule);

		} catch(Exception $e) {
			$result["data"] = $e->getMessage();
			$result["status"] = 503;
		}
		return print(json_encode($result));
	}
	/**
	 *	Fetches top level result from the passive check using Livestatus.
	 *	@return string
	 */
	public function check()
	{
		$this->auto_render = false;
		$result = array("status" => 200, "data" => false);
		try {
			if (!isset($_GET["service"])) {
				throw new Exception("You have to pass a service argument");
			}
			if (strpos($_GET["service"], '&') === false) {
				throw new Exception("Illegal service argument");
			}
			$result['data'] = false !== ServicePool_Model::fetch_by_key(str_replace('&', ';', $_GET['service']));
		} catch(Exception $e) {
			$result["data"] = $e->getMessage();
			$result["status"] = 503;
		}
		return print(json_encode($result));
	}

	/**
	 * Save created, edited and deleted trees.
	 * @return string
	 */
	public function sync() {
		if (nacoma::allowed() === false) {
			header("HTTP/1.0 401 Unauthorized");
			exit;
		}
		$output = "";
		$this->auto_render = false;
		try {
			$nacomaapi = new NacomaApi_Model();

			$uuid = uniqid();
			if (isset($_POST["update_list"])) {
				$update_list = json_decode($_POST["update_list"], true);
				foreach ($update_list as $rule_name => $rule_json) {
					Synergy_Model::saveRule($rule_json, $rule_name, $uuid);
				}
			}

			if (isset($_POST["del_list"])) {
				Synergy_Model::deleteRulesList($_POST["del_list"], $uuid);
			}
			Synergy_Model::commitActions($uuid);

			if (isset($_POST["add"]) or isset($_POST["del"]) or isset($_POST["rename"])) {
				if (isset($_POST["add"])) {
					$add = json_decode($_POST["add"], true);
					$nacomaapi->create_services($add, $output);
				}

				if (isset($_POST["del"])) {
					$del = json_decode($_POST["del"], true);
					$nacomaapi->delete_services($del, $output);
				}

				if (isset($_POST["rename"])) {
					$ren = json_decode($_POST["rename"], true);
					$nacomaapi->rename_services($ren, $output);
				}
				$nacomaapi->saveconfig($output);
			}
		} catch (Exception $e) {
			return print(json_encode(array("result" => false, "msg" => $e->getMessage())));
		}

		return print(json_encode(array("result" => true, "timestamp" => time())));
	}
}
