local _G = _G
module(...)
at_least = {
	name = "At least",
	tag = "synergy_group",
	order = 4,
	description = "Specify number or percent of services you need to be OK and CRITICAL",
	rule = function(items, count_ok, count_wr, unit)
		if count_wr >= count_ok then
			return {{ status = 3, msg = "Rule is not properly configured. OK threshold must be greater than WARNING threshold." }}
		end
		local c_ok = 0
		local c_wr = 0
		local c_cr = 0
		local c_un = 0
		local n = 0
		local res_st = 2
		local res_msg = "Require %s OK(s), got: %s OK(s), %s WARNING(s), %s CRITICAL(s), %s UNKNOWN(s)"
		local state_map = { "OK", "WARNING", "CRITICAL" }
		for i,value in _G.ipairs(items) do
			if value.status == 0 then
				c_ok = c_ok + 1
			elseif value.status == 1 then
				c_wr = c_wr + 1
			elseif value.status == 2 then
				c_cr = c_cr + 1
			elseif value.status == 3 then
				c_un = c_un + 1
			else
				return {{ status = 3, msg = value.name.." has a status that can't be interpreted by the At Least rule. Exiting with unknown status."}}
			end
			n = n + 1
		end
		if n == 0 then return {{ status = 3, msg = "No state, default state UNKNOWN" }} end
		if unit == "%" then
			local P_ok, P_wr, P_cr, P_un
			P_ok =_G.math.ceil(100*c_ok/n)
			P_wr =_G.math.ceil(100*c_wr/n)
			P_cr =_G.math.ceil(100*c_cr/n)
			P_un =_G.math.ceil(100*c_un/n)
			res_msg = _G.string.format(res_msg, count_ok.."%", P_ok.."%", P_wr.."%", P_cr.."%", P_un.."%")
			if P_ok >= count_ok then res_st = 0
			elseif P_ok >= count_wr then res_st = 1
			end
		else
			if n < count_ok then return {{ status = 2, msg = "Not enough subelements to be OK, defined - state CRITICAL" }} end
			if c_ok >= count_ok then res_st = 0
			elseif c_ok >= count_wr then res_st = 1
			end
			res_msg = _G.string.format(res_msg, count_ok, c_ok, c_wr, c_cr, c_un)
		end
		return {{ status = res_st, msg = res_msg }}
	end,
	params = {
		{
			validation = {
				required = true,
				number = true,
				gt = { true, "count_wr", "WARNING threshold" },
				percent = true,
				min = { 0 }
			},
			name = "count_ok",
			label = "OK threshold",
			type = "number",
			hint = "Count of OKs to remain OK"
		},
		{
			validation = {
				required = true,
				number = true,
				lt = { true, "count_ok", "OK threshold" },
				percent = true,
				min = { 0 }
			},
			name = "count_wr",
			label = "WARNING threshold",
			type = "number",
			hint = "Count of OKs to remain WARNING"
		},
		{
			validation = {
				required = true
			},
			name = "unit",
			label = "Unit",
			type = "enum",
			hint = "Unit for the above value (percentage or actual number)",
			values = {"%", "num"}
		}
	}
}
