local _G = _G
module(...)
host_services = {
	name = "Host Services",
	tag = "nagios",
	order = 9,
	description = "Counts as one object and returns worst state among the hosts services.",
	hint = "Unfold services from host",
	links = {
		{ href = "/extinfo/details/host/{{nagios_host_services}}", title = "host details" },
		{ href = "/configuration/configure/host/{{nagios_host_services}}", title = "host configuration" },
		{ href = "/status/service/{{nagios_host_services}}", title = "host services" }
	},
	rule = function(items, name, type)
		local ret = {}
		local h = _G.host(name)
		local ss
		local countsrv, srv
		if h ~= nil then
			countsrv = h.total_services
			ss = h.services
		end
		if h == nil or countsrv == 0 then
			return {{
				status = 3, 
				msg = "Object not found", 
				service_contacts = "*",
				service_contact_groups = "*",
				host_contacts = "*",
				host_contact_groups = "*"
			}}
		end
		local state = 0
		local state_change
		for i,desc in _G.ipairs(ss) do
			srv = h:service(desc)
			if (type == "all") or (srv.state_type == 1) then
				if state < srv.state then
					state = srv.state
					state_change = srv.last_state_change
				end
			else
				if state < srv.last_hard_state then
					state = srv.last_hard_state
					state_change = srv.last_hard_state_change
				end
			end
		end
		_G.table.insert(ret, {
			status = state,
			msg = "Number of services on host: "..h.num_services,
			last_check = srv.last_check,
			last_state_change = state_change,
			service_contacts = srv.contacts,
			service_contact_groups = srv.contact_groups,
			host_contacts = "*",
			host_contact_groups = "*"
		})
		return ret
	end,
	params = {
		{
			validation = {
				required = true,
				unique_in_scope = true
			},
			name = "nagios_host_services",
			label = "Host",
			type = "nagios_host",
			hint = "Related host name"
		}
	}
}
