#!/usr/bin/python

import sys, os, glob, re, urllib

def error(s):
	print(s)
	sys.exit(1)

class bps_fix:
	forbidden = '~!$%^&*|\'\"<>?,()=\\;'
	subst = '-'

	def __init__(self, base_path='/opt/synergy/etc/bps'):
		self.target_files = {}
		self.target_count = {}
		self.inspected = {}
		self.nochange = {}
		self.remove = {}
		self.base_path = base_path
		self.ok = 0
		self.total = 0
		self.verbose = 0

	def php_urlencode(self, s):
		# because php is stupid...
		return urllib.quote(s).replace('%20', '+')

	def add_target(self, src, dest, buf=False):
		if buf:
			oldbuf = self.target_files.get(dest, '')
			if oldbuf == buf:
				if src == dest:
					dupe = self.inspected[dest]
					self.inspected[dest] = src
				else:
					dupe = src
				self.remove[dupe] = True
				print("  Dupe: %s" % os.path.basename(dupe))
				return
			self.target_files[dest] = buf
		if not self.target_count.get(dest):
			self.target_count[dest] = 1
		else:
			self.target_count[dest] += 1
			# the error will be printed later, when we know how many
			# files link to the same name
		self.inspected[dest] = src


	def fix_string(self, s):
		# for some reason, backslashes can't be regex-substituted.
		s = s.replace('\\', '-')
		return re.sub('[%s]' % self.forbidden, self.subst, s).strip()


	def fix_file(self, path):
		pristine = True
		first_name = False
		directory = os.path.dirname(path)
		fname = os.path.basename(path)
		udec = urllib.unquote(fname)
		rp = self.fix_string(udec)
		self.total += 1
		f = open(path, 'r')
		i = 0
		lines = []
		for line in f:
			i += 1
			l = line.strip()
			if not l.startswith('name'):
				lines.append(line)
				continue
			first_quote = l.find('"')
			if first_quote < 0:
				break
			first_quote += 1
			second_quote = l[first_quote:].find('"')
			if second_quote < 0:
				# seriously malformed, by the looks of it
				break
			second_quote += first_quote
			orig_name = l[first_quote:second_quote]
			name = self.fix_string(orig_name.strip())
			if not first_name:
				first_name = name
			if name == orig_name:
				lines.append(line)
				continue

			# One name needs replacing, so let's go to town
			pristine = False

			l = line.replace(orig_name, name)
			lines.append(l)

		f.close()

		if not first_name:
			error("No \"name\" variable found. What the hells?")
			return False

		real_path = "%s/%s.lua" % (os.path.dirname(path), self.php_urlencode(first_name))
		if real_path == path and pristine:
			self.ok += 1
			if self.verbose:
				print("  OK: %s" % os.path.basename(path))
		else:
			print("  Needs work: %s" % os.path.basename(path))
		self.add_target(path, real_path, ''.join(lines))
		if pristine:
			self.nochange[real_path] = True


	def fix_files(self, path='/opt/synergy/etc/bps'):
		print("Inspecting %s/*.lua" % path)
		res = glob.glob('%s/*.lua' % path)
		for path in res:
			self.fix_file(path)


	def validate(self):
		errors = 0
		for path, count in self.target_count.items():
			if count == 1:
				continue
			print("%d files designated to %s" % (count, path))
			errors += 1
		if errors:
			print("Encountered errors. Aborting")
			sys.exit(1)


	def write_fixed_files(self):
		if not len(self.remove) and len(self.nochange) == len(self.target_files):
			return
		print("Applying changes")
		for path in self.remove.keys():
			print("  removing %s (dupe)" % os.path.basename(path))
			os.unlink(path)
		for dest, src in self.inspected.items():
			if src == dest:
				continue
			print("  renaming %s -> %s" % (os.path.basename(src), os.path.basename(dest)))
			os.rename(src, dest)
		for p, buf in self.target_files.items():
			if self.nochange.get(p):
				continue
			print("  updating %s (content fixed)" % os.path.basename(p))
			f = open(p, 'w')
			f.write(buf)
			f.close()
		return

bfix = bps_fix('/opt/synergy/etc/bps')
bfix.fix_files()
if bfix.total == bfix.ok:
	print("All %d rules check out ok" % (bfix.ok))
else:
	bfix.validate()
	bfix.write_fixed_files()
	print("%d/%d rules fixed (%d ok)" % (bfix.total - bfix.ok, bfix.total, bfix.ok))
